import unittest

import numpy as np

from sigllm.primitives.postprocessing import aggregate_rolling_window


class FormatAsStringTest(unittest.TestCase):

    def test_aggregate_rolling_window_shape_1_1_5(self):

        data = np.array([[
            [1, 2, 3, 4, 5]
        ]])
        expected = np.array(
            [1, 2, 3, 4, 5]
        )

        output = aggregate_rolling_window(data)

        np.testing.assert_array_equal(output, expected)

    def test_aggregate_rolling_window_shape_1_3_5(self):

        data = np.array([[
            [1, 2, 3, 4, 5],
            [2, 3, 4, 5, 6],
            [3, 4, 5, 6, 7]
        ]])
        expected = np.array(
            [2, 3, 4, 5, 6]
        )

        output = aggregate_rolling_window(data)

        np.testing.assert_array_equal(output, expected)

    def test_aggregate_rolling_window_shape_1_5_1(self):

        data = np.array([[
            [1],
            [2],
            [3],
            [4],
            [5]
        ]])
        expected = np.array(
            [3]
        )

        output = aggregate_rolling_window(data)

        np.testing.assert_array_equal(output, expected)

    def test_aggregate_rolling_window_shape_3_1_5(self):

        data = np.array([
            [[1, 2, 3, 4, 5]],
            [[2, 3, 4, 5, 6]],
            [[3, 4, 5, 6, 7]]
        ])
        expected = np.array(
            [1, 2, 3, 4, 5, 6, 7]
        )

        output = aggregate_rolling_window(data)

        np.testing.assert_array_equal(output, expected)

    def test_aggregate_rolling_window_shape_3_2_5(self):

        data = np.array([
            [[1, 2, 3, 4, 5], [2, 3, 4, 5, 6]],
            [[2, 3, 4, 5, 6], [3, 4, 5, 6, 7]],
            [[3, 4, 5, 6, 7], [4, 5, 6, 7, 8]]
        ])
        expected = np.array(
            [1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 7.5]
        )

        output = aggregate_rolling_window(data)

        np.testing.assert_array_equal(output, expected)



# -*- coding: utf-8 -*-

"""
Data preprocessing module.

This module contains functions that prepare timeseries for a language model.
"""

import numpy as np


def rolling_window_sequences(X, index, window_size, step_size):
    """Create rolling window sequences out of time series data.

    The function creates an array of sequences by rolling over the input sequence.

    Args:
        X (ndarray):
            The sequence to iterate over.
        index (ndarray):
            Array containing the index values of X.
        window_size (int):
            Length of window.
        step_size (int):
            Indicating the number of steps to move the window forward each round.

    Returns:
        ndarray, ndarray:
            * rolling window sequences.
            * first index value of each input sequence.
    """
    out_X = list()
    X_index = list()

    start = 0
    max_start = len(X) - window_size + 1
    while start < max_start:
        end = start + window_size
        out_X.append(X[start:end])
        X_index.append(index[start])
        start = start + step_size

    return np.asarray(out_X), np.asarray(X_index)


def sig2str(values, sep=',', space=False, decimal=0, rescale=True):
    """Convert a signal to a string.

    Convert a 1-dimensional time series into text by casting and rescaling it
    to nonnegative integer values then into a string (optional).

    Args:
        values (numpy.ndarray):
            A sequence of signal values.
        sep (str):
            String to separate each element in values. Default to `","`.
        space (bool):
            Whether to add space between each digit in the result. Default to `False`.
        decimal (int):
            Number of decimal points to keep from the float representation. Default to `0`.
        rescale(bool):
            Whether to rescale the time series. Default to `True`

    Returns:
        str:
            Text containing the elements of `values`.
    """
    sign = 1 * (values >= 0) - 1 * (values < 0)
    values = np.abs(values)

    sequence = sign * (values * 10**decimal).astype(int)

    # Rescale all elements to be nonnegative
    if rescale:
        sequence = sequence - min(sequence)

    res = sep.join([str(num) for num in sequence])
    if space:
        res = ' '.join(res)

    return res



# -*- coding: utf-8 -*-

"""
Result post-processing module.

This module contains functions that help convert model responses back to indices and timestamps.
"""
import numpy as np


def str2sig(text, sep=',', decimal=0):
    """Convert a text string to a signal.

    Convert a string containing digits into an array of numbers.

    Args:
        text (str):
            A string containing signal values.
        sep (str):
            String that was used to separate each element in text, Default to `","`.
        decimal (int):
            Number of decimal points to shift each element in text to. Default to `0`.

    Returns:
        numpy.ndarray:
            A 1-dimensional array containing parsed elements in `text`.
    """
    # Remove all characters from text except the digits and sep and decimal point
    text = ''.join(i for i in text if (i.isdigit() or i == sep or i == '.'))
    values = np.fromstring(text, dtype=float, sep=sep)
    return values * 10**(-decimal)


def str2idx(text, len_seq, sep=','):
    """Convert a text string to indices.

    Convert a string containing digits into an array of indices.

    Args:
        text (str):
            A string containing indices values.
        len_seq (int):
            The length of processed sequence
        sep (str):
            String that was used to separate each element in text, Default to `","`.

    Returns:
        numpy.ndarray:
            A 1-dimensional array containing parsed elements in `text`.
    """
    # Remove all characters from text except the digits and sep
    text = ''.join(i for i in text if (i.isdigit() or i == sep))

    values = np.fromstring(text, dtype=int, sep=sep)

    # Remove indices that exceed the length of sequence
    values = values[values < len_seq]
    return values


def get_anomaly_list_within_seq(res_list, alpha=0.5):
    """Get the final list of anomalous indices of a sequence

    Choose anomalous index in the sequence based on multiple LLM responses

    Args:
        res_list (List[numpy.ndarray]):
            A list of 1-dimensional array containing anomous indices output by LLM
        alpha (float):
            Percentage of votes needed for an index to be deemed anomalous. Default: 0.5

    Returns:
        numpy.ndarray:
            A 1-dimensional array containing final anomalous indices
    """
    min_vote = np.ceil(alpha * len(res_list))

    flattened_res = np.concatenate(res_list)

    unique_elements, counts = np.unique(flattened_res, return_counts=True)

    final_list = unique_elements[counts >= min_vote]

    return final_list


def merge_anomaly_seq(anomalies, start_indices, window_size, step_size, beta=0.5):
    """Get the final list of anomalous indices of a sequence when merging all rolling windows

    Args:
        anomalies (List[numpy.ndarray]):
            A list of 1-dimensional array containing anomous indices of each window
        start_indices (numpy.ndarray):
            A 1-dimensional array contaning the first index of each window
        window_size (int):
            Length of each window
        step_size (int):
            Indicating the number of steps the window moves forward each round.
        beta (float):
            Percentage of containing windows needed for index to be deemed anomalous. Default: 0.5

    Return:
        numpy.ndarray:
            A 1-dimensional array containing final anomalous indices
    """
    anomalies = [arr + first_idx for (arr, first_idx) in zip(anomalies, start_indices)]

    min_vote = np.ceil(beta * window_size / step_size)

    flattened_res = np.concatenate(anomalies)

    unique_elements, counts = np.unique(flattened_res, return_counts=True)

    final_list = unique_elements[counts >= min_vote]

    return np.sort(final_list)


def idx2time(sequence, idx_list):
    """Convert list of indices into list of timestamp

    Args:
        sequence (pandas.Dataframe):
            Signal with timestamps and values
        idx_list (numpy.ndarray):
            A 1-dimensional array of indices

    Returns:
        numpy.ndarray:
            A 1-dimensional array containing timestamps
    """
    return sequence.iloc[idx_list].timestamp.to_numpy()



# -*- coding: utf-8 -*-

import numpy as np
from pytest import fixture

from sigllm.primitives.prompting.data import rolling_window_sequences, sig2str


@fixture
def integers():
    return np.array([1, 2, 3, 4, 5, 6, 7, 8, 9])


@fixture
def floats():
    return np.array([
        1.283,
        2.424,
        3.213,
        4.583,
        5.486,
        6.284,
        7.297,
        8.023,
        9.786
    ])


@fixture
def negatives():
    return np.array([
        -2.5,
        -1.5,
        0,
        1.5,
        2.5,
    ])


@fixture
def indices():
    return np.array([0, 1, 2, 3, 4, 5, 6])


@fixture
def values():
    return np.array([0.555, 2.345, 1.501, 5.903, 9.116, 3.068, 4.678])


@fixture
def window_size():
    return 3


@fixture
def step_size():
    return 1


def test_sig2str(integers):
    expected = '0,1,2,3,4,5,6,7,8'

    result = sig2str(integers)

    assert result == expected


def test_sig2str_noscale(integers):
    expected = '1,2,3,4,5,6,7,8,9'

    result = sig2str(integers, rescale=False)

    assert result == expected


def test_sig2str_decimal(integers):
    expected = '0,100,200,300,400,500,600,700,800'

    result = sig2str(integers, decimal=2)

    assert result == expected


def test_sig2str_sep(integers):
    expected = '0|1|2|3|4|5|6|7|8'

    result = sig2str(integers, sep='|')

    assert result == expected


def test_sig2str_space(integers):
    expected = '0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8'

    result = sig2str(integers, space=True)

    assert result == expected


def test_sig2str_float(floats):
    expected = '0,1,2,3,4,5,6,7,8'

    result = sig2str(floats)

    assert result == expected


def test_sig2str_float_decimal(floats):
    expected = '0,114,193,330,420,500,601,674,850'

    result = sig2str(floats, decimal=2)

    assert result == expected


def test_sig2str_negative_decimal(negatives):
    expected = '0,10,25,40,50'

    result = sig2str(negatives, decimal=1)

    assert result == expected


def test_rolling_window_sequences(values, indices, window_size, step_size):
    expected = (np.array([[0.555, 2.345, 1.501],
                          [2.345, 1.501, 5.903],
                          [1.501, 5.903, 9.116],
                          [5.903, 9.116, 3.068],
                          [9.116, 3.068, 4.678], ]),
                np.array([0, 1, 2, 3, 4]))

    result = rolling_window_sequences(values, indices, window_size, step_size)

    if len(result) != len(expected):
        raise AssertionError("Tuples has different length")

    for arr1, arr2 in zip(result, expected):
        np.testing.assert_equal(arr1, arr2)



# -*- coding: utf-8 -*-

"""
GPT model module.

This module contains functions that are specifically used for GPT models
"""
import os

from openai import OpenAI


def load_system_prompt(file_path):
    with open(file_path) as f:
        system_prompt = f.read()
    return system_prompt


CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))

ZERO_SHOT_FILE = 'gpt_system_prompt_zero_shot.txt'
ONE_SHOT_FILE = 'gpt_system_prompt_one_shot.txt'

ZERO_SHOT_DIR = os.path.join(CURRENT_DIR, "..", "template", ZERO_SHOT_FILE)
ONE_SHOT_DIR = os.path.join(CURRENT_DIR, "..", "template", ONE_SHOT_FILE)


GPT_model = "gpt-4"  # "gpt-4-0125-preview" #  #  #"gpt-3.5-turbo" #
client = OpenAI()


def get_gpt_model_response(message, gpt_model=GPT_model):
    completion = client.chat.completions.create(
        model=gpt_model,
        messages=message,
    )
    return completion.choices[0].message.content


def create_message_zero_shot(seq_query, system_prompt_file=ZERO_SHOT_DIR):
    messages = []

    messages.append({"role": "system", "content": load_system_prompt(system_prompt_file)})

    # final prompt
    messages.append({"role": "user", "content": f"Sequence: {seq_query}"})
    return messages


def create_message_one_shot(seq_query, seq_ex, ano_idx_ex, system_prompt_file=ONE_SHOT_DIR):
    messages = []

    messages.append({"role": "system", "content": load_system_prompt(system_prompt_file)})

    # one shot
    messages.append({"role": "user", "content": f"Sequence: {seq_ex}"})
    messages.append({"role": "assistant", "content": ano_idx_ex})

    # final prompt
    messages.append({"role": "user", "content": f"Sequence: {seq_query}"})
    return messages



# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
from pytest import fixture

from sigllm.primitives.prompting.anomalies import (
    get_anomaly_list_within_seq, idx2time, merge_anomaly_seq, str2idx, str2sig,)


@fixture
def text():
    return '1,2,3,4,5,6,7,8,9'


@fixture
def text_1():
    return 'Result: 1 2 3, 2 3 4,'


@fixture
def text_float():
    return 'Result: 1.23, 2.34,'


@fixture
def anomaly_list_within_seq():
    return [np.array([2, 3, 7, 9]),
            np.array([5]),
            np.array([2, 5]),
            np.array([8, 9])]


@fixture
def anomaly_list_across_seq():
    return [np.array([0]),
            np.array([1, 2]),
            np.array([0, 2]),
            np.array([1, 2]),
            np.array([1])]


@fixture
def first_indices():
    return np.array([0, 1, 2, 3, 4])


@fixture
def window_size():
    return 3


@fixture
def step_size():
    return 1


@fixture
def signal():
    d = {'timestamp': [1222819200, 1222840800, 1222862400, 1222884000,
                       1222905600], 'value': [-1.0, -1.0, -1.0, -1.0, -1.0]}
    return pd.DataFrame(data=d)


@fixture
def idx_list():
    return np.array([0, 1, 3])


def test_str2sig(text_float):
    expected = np.array([0.123, 0.234])

    result = str2sig(text_float, decimal=1)

    np.testing.assert_allclose(result, expected, rtol=1e-15, atol=0)


def test_str2idx(text):
    expected = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9])

    result = str2idx(text, len_seq=20)

    np.testing.assert_equal(result, expected)


def test_str2idx_spurious(text_1):
    expected = np.array([123, 234])

    result = str2idx(text_1, len_seq=500)

    np.testing.assert_equal(result, expected)


def test_str2idx_overflow(text):
    expected = np.array([1, 2, 3, 4, 5, 6, 7])

    result = str2idx(text, len_seq=8)

    np.testing.assert_equal(result, expected)


def test_get_anomaly_list_within_seq(anomaly_list_within_seq):
    expected = np.array([2, 5, 9])

    result = get_anomaly_list_within_seq(anomaly_list_within_seq)

    np.testing.assert_equal(result, expected)


def test_merge_anomaly_seq(anomaly_list_across_seq, first_indices, window_size, step_size):
    expected = np.array([2, 4, 5])

    result = merge_anomaly_seq(anomaly_list_across_seq, first_indices, window_size, step_size)

    np.testing.assert_equal(result, expected)


def test_idx2time(signal, idx_list):
    expected = np.array([1222819200, 1222840800, 1222884000])

    result = idx2time(signal, idx_list)

    np.testing.assert_equal(result, expected)



# -*- coding: utf-8 -*-

"""Top-level package for sigllm."""

__author__ = 'MIT Data To AI Lab'
__email__ = 'dailabmit@gmail.com'
__version__ = '0.0.1'

import os

from sigllm.core import SigLLM  # noqa: F401

_BASE_PATH = os.path.abspath(os.path.dirname(__file__))
MLBLOCKS_PRIMITIVES = os.path.join(_BASE_PATH, 'primitives', 'jsons')
MLBLOCKS_PIPELINES = tuple([
    os.path.join(_BASE_PATH, 'pipelines', 'prompter'),
    os.path.join(_BASE_PATH, 'pipelines', 'detector')
])



# -*- coding: utf-8 -*-

"""
Main module.

SigLLM is an extension to Orion's core module
"""
import logging
from typing import Union

import pandas as pd
from mlblocks import MLPipeline
from orion import Orion

LOGGER = logging.getLogger(__name__)

INTERVAL_PRIMITIVE = "mlstars.custom.timeseries_preprocessing.time_segments_aggregate#1"
DECIMAL_PRIMITIVE = "sigllm.primitives.transformation.Float2Scalar#1"
WINDOW_SIZE_PRIMITIVE = "sigllm.primitives.forecasting.custom.rolling_window_sequences#1"


class SigLLM(Orion):
    """SigLLM Class.

    The SigLLM Class provides the main anomaly detection functionalities
    of SigLLM and is responsible for the interaction with the underlying
    MLBlocks pipelines.

    Args:
        pipeline (str, dict or MLPipeline):
            Pipeline to use. It can be passed as:
                * An ``str`` with a path to a JSON file.
                * An ``str`` with the name of a registered pipeline.
                * An ``MLPipeline`` instance.
                * A ``dict`` with an ``MLPipeline`` specification.
        interval (int):
            Number of time points between one sample and another.
        decimal (int):
            Number of decimal points to keep from the float representation.
        window_size (int):
            Size of the input window.
        hyperparameters (dict):
            Additional hyperparameters to set to the Pipeline.
    """
    DEFAULT_PIPELINE = 'mistral_detector'

    def _augment_hyperparameters(self, primitive, key, value):
        if self._hyperparameters is None:
            self._hyperparameters = {
                primitive: {}
            }
        else:
            if primitive not in self._hyperparameters:
                self._hyperparameters[primitive] = {}

        if value:
            self._hyperparameters[primitive][key] = value

    def __init__(self, pipeline: Union[str, dict, MLPipeline] = None, interval: int = None,
                 decimal: int = None, window_size: int = None, hyperparameters: dict = None):
        self._pipeline = pipeline or self.DEFAULT_PIPELINE
        self._hyperparameters = hyperparameters
        self._mlpipeline = self._get_mlpipeline()
        self._fitted = False

        self.interval = interval
        self.decimal = decimal
        self.window_size = window_size

        self._augment_hyperparameters(INTERVAL_PRIMITIVE, 'interval', interval)
        self._augment_hyperparameters(DECIMAL_PRIMITIVE, 'decimal', decimal)
        self._augment_hyperparameters(WINDOW_SIZE_PRIMITIVE, 'window_size', window_size)

    def __repr__(self):
        if isinstance(self._pipeline, MLPipeline):
            pipeline = '\n'.join(
                '    {}'.format(primitive) for primitive in self._pipeline.to_dict()['primitives'])

        elif isinstance(self._pipeline, dict):
            pipeline = '\n'.join(
                '    {}'.format(primitive) for primitive in self._pipeline['primitives'])

        else:
            pipeline = '    {}'.format(self._pipeline)

        hyperparameters = None
        if self._hyperparameters is not None:
            hyperparameters = '\n'.join(
                '    {}: {}'.format(step, value) for step, value in self._hyperparameters.items())

        return (
            'SigLLM:\n{}\n'
            'hyperparameters:\n{}\n'
        ).format(
            pipeline,
            hyperparameters
        )

    def detect(self, data: pd.DataFrame, visualization: bool = False, **kwargs) -> pd.DataFrame:
        """Detect anomalies in the given data..

        If ``visualization=True``, also return the visualization
        outputs from the MLPipeline object.

        Args:
            data (DataFrame):
                Input data, passed as a ``pandas.DataFrame`` containing
                exactly two columns: timestamp and value.
            visualization (bool):
                If ``True``, also capture the ``visualization`` named
                output from the ``MLPipeline`` and return it as a second
                output.

        Returns:
            DataFrame or tuple:
                If visualization is ``False``, it returns the events
                DataFrame. If visualization is ``True``, it returns a
                tuple containing the events DataFrame followed by the
                visualization outputs dict.
        """
        if not self._fitted:
            self._mlpipeline = self._get_mlpipeline()

        result = self._detect(self._mlpipeline.fit, data, visualization, **kwargs)
        self._fitted = True

        return result



import unittest

import numpy as np
import pytest

from sigllm.primitives.transformation import (
    Float2Scalar, Scalar2Float, _from_string_to_integer, format_as_integer, format_as_string,)


class FormatAsStringTest(unittest.TestCase):

    def test_format_as_string_default(self):

        data = np.array([[
            1, 2, 3, 4, 5
        ]])
        expected = np.array([
            '1,2,3,4,5'
        ])

        output = format_as_string(data)

        np.testing.assert_array_equal(output, expected)

    def test_format_as_string_space(self):

        data = np.array([[
            1, 2, 3, 4, 5
        ]])
        expected = np.array([
            '1 , 2 , 3 , 4 , 5'
        ])

        output = format_as_string(data, space=True)

        np.testing.assert_array_equal(output, expected)

    def test_format_as_string_sep(self):

        data = np.array([[
            1, 2, 3, 4, 5
        ]])
        expected = np.array([
            '1.2.3.4.5'
        ])

        output = format_as_string(data, sep='.')

        assert output == expected

    def test_format_as_string_decimal(self):

        data = np.array([[
            100, 200, 300, 400, 500
        ]])
        expected = np.array([
            '100,200,300,400,500'
        ])

        output = format_as_string(data)

        assert output == expected


class FromStringToIntegerTest(unittest.TestCase):

    def test__from_string_to_integer_default(self):

        data = '1,2,3,4,5'

        expected = np.array([
            1, 2, 3, 4, 5
        ])

        output = _from_string_to_integer(data)

        np.testing.assert_equal(output, expected)

    def test__from_string_to_integer_trunc_all(self):

        data = '1,2,3,4,5'

        expected = np.array([
            1, 2, 3, 4, 5
        ])

        output = _from_string_to_integer(data, trunc=5)

        np.testing.assert_equal(output, expected)

    def test__from_string_to_integer_trunc_two(self):

        data = '134,252,3,412,51'

        expected = np.array([
            134, 252
        ])

        output = _from_string_to_integer(data, trunc=2)

        np.testing.assert_equal(output, expected)

    def test__from_string_to_integer_ignore(self):

        data = '1 , 2!e , 3,4,5'

        expected = np.array([
            1, 3, 4, 5
        ])

        output = _from_string_to_integer(data, errors='ignore')

        np.testing.assert_equal(output, expected)

    def test__from_string_to_integer_filter(self):

        data = '1 , 2!e , 3,4,5'

        expected = np.array([
            1, 2, 3, 4, 5
        ])

        output = _from_string_to_integer(data, errors='filter')

        np.testing.assert_equal(output, expected)

    def test__from_string_to_integer_coerce(self):

        data = '1 , 2!e , 3,4,5'

        expected = np.array([
            1, np.nan, 3, 4, 5
        ])

        output = _from_string_to_integer(data, errors='coerce')

        np.testing.assert_equal(output, expected)

    def test__from_string_to_integer_raise(self):

        data = '1 , 2!e , 3,4,5'

        with pytest.raises(ValueError):
            _from_string_to_integer(data, errors='raise')

    def test__from_string_to_integer_error(self):

        data = '1 , 2 , 3,4,5'

        with pytest.raises(KeyError):
            _from_string_to_integer(data, errors='unknown')


def test_format_as_integer_one():
    data = ['1,2,3,4,5']

    with pytest.raises(ValueError):
        format_as_integer(data)


def test_format_as_integer_list():
    data = [['1,2,3,4,5']]

    expected = np.array([[[
        1, 2, 3, 4, 5
    ]]])

    output = format_as_integer(data)

    np.testing.assert_equal(output, expected)


def test_format_as_integer_2d_shape_mismatch():
    data = [['1,2,3,4,5'], ['1, 294., 3 , j34,5'], ['!232, 23,3,4,5']]

    expected = np.array([
        [np.array([1., 2, 3, 4, 5])],
        [np.array([1., 3, 5])],
        [np.array([23., 3, 4, 5])]
    ], dtype=object)

    output = format_as_integer(data)

    for out, exp in list(zip(output, expected)):
        for o, e in list(zip(out, exp)):
            np.testing.assert_equal(o, e)


def test_format_as_integer_2d_trunc():
    data = [['1,2,3,4,5'], ['1,294.,3,j34,5'], ['!232, 23,3,4,5']]

    expected = np.array([
        [[1, 2]],
        [[1, 3]],
        [[23, 3]]
    ])

    output = format_as_integer(data, trunc=2)

    np.testing.assert_equal(output, expected)


def test_format_as_integer_3d():
    data = [
        ['1,2,3,4,5', '6,7,8,9,10'],
        ['11,12,13,14,15', '16,17,18,19,20']
    ]

    expected = np.array([
        [[1, 2, 3, 4, 5], [6, 7, 8, 9, 10]],
        [[11, 12, 13, 14, 15], [16, 17, 18, 19, 20]]
    ])

    output = format_as_integer(data)

    np.testing.assert_equal(output, expected)


class Float2ScalarTest(unittest.TestCase):

    def test_transform_default(self):
        converter = Float2Scalar()

        data = np.array([
            1.05, 2., 3.1, 4.8342, 5, 0
        ])
        expected = np.array([
            105, 200, 310, 483, 500, 0
        ])
        print(converter)

        converter.fit(data)
        output, minimum, decimal = converter.transform(data)

        assert converter.decimal == 2
        assert converter.rescale is True
        assert converter.minimum == 0

        np.testing.assert_array_equal(output, expected)

    def test_transform_decimal_zero(self):
        converter = Float2Scalar(decimal=0)

        data = np.array([
            1.05, 2., 3.1, 4.8342, 5, 0
        ])
        expected = np.array([
            1, 2, 3, 4, 5, 0
        ])

        converter.fit(data)
        output, minimum, decimal = converter.transform(data)

        assert converter.decimal == 0
        assert converter.rescale is True
        assert converter.minimum == 0

        np.testing.assert_array_equal(output, expected)

    def test_transform_minimum_not_zero(self):
        converter = Float2Scalar()

        data = np.array([
            1.05, 2., 3.1, 4.8342, 5
        ])
        expected = np.array([
            0, 95, 204, 378, 395
        ])

        converter.fit(data)
        output, minimum, decimal = converter.transform(data)

        assert converter.decimal == 2
        assert converter.rescale is True
        assert converter.minimum == 1.05

        np.testing.assert_array_equal(output, expected)

    def test_transform_rescale_false(self):
        converter = Float2Scalar(rescale=False)

        data = np.array([
            1.05, 2., 3.1, 4.8342, 5
        ])
        expected = np.array([
            105, 200, 310, 483, 500
        ])

        converter.fit(data)
        output, minimum, decimal = converter.transform(data)

        assert converter.decimal == 2
        assert converter.rescale is False
        assert converter.minimum == 1.05

        np.testing.assert_array_equal(output, expected)

    def test_transform_negative(self):
        converter = Float2Scalar()

        data = np.array([
            1.05, 2., 3.1, 4.8342, 5, -2
        ])
        expected = np.array([
            305, 400, 509, 683, 700, 0
        ])

        converter.fit(data)
        output, minimum, decimal = converter.transform(data)

        assert converter.decimal == 2
        assert converter.rescale is True
        assert converter.minimum == -2

        np.testing.assert_array_equal(output, expected)

    def test_transform_fit_different(self):
        converter = Float2Scalar()

        data = np.array([
            1.05, 2., 3.1, 4.8342, 5
        ])
        expected = np.array([
            55, 150, 260, 433, 450
        ])

        converter.fit([7, 3, 0.5])
        output, minimum, decimal = converter.transform(data)

        assert converter.decimal == 2
        assert converter.rescale is True
        assert converter.minimum == 0.5

        np.testing.assert_array_equal(output, expected)


class Scalar2FloatTest(unittest.TestCase):

    def test_transform_default(self):
        converter = Scalar2Float()

        data = np.array([
            105, 200, 310, 483, 500, 0
        ])
        expected = np.array([
            1.05, 2., 3.1, 4.83, 5, 0
        ])

        output = converter.transform(data)

        np.testing.assert_array_equal(output, expected)

    def test_transform_decimal_zero(self):
        converter = Scalar2Float()

        data = np.array([
            1, 2, 3, 4, 5, 0
        ])
        expected = np.array([
            1., 2., 3., 4., 5., 0.
        ])

        output = converter.transform(data, decimal=0)

        np.testing.assert_array_equal(output, expected)

    def test_transform_minimum_not_zero(self):
        converter = Scalar2Float()

        data = np.array([
            105, 200, 310, 483, 500, 0
        ])
        expected = np.array([
            0.05, 1., 2.1, 3.83, 4., -1.
        ])

        output = converter.transform(data, minimum=-1)

        np.testing.assert_allclose(output, expected)


def test_float2scalar_scalar2float_integration():
    decimal = 2
    rescale = True

    float2scalar = Float2Scalar(decimal, rescale)

    data = np.array([
        1.05, 2., 3.1, 4.8342, 5, -1
    ])

    expected = np.array([
        1.05, 2., 3.10, 4.83, 5., -1.
    ])

    float2scalar.fit(data)
    transformed, minimum, decimal = float2scalar.transform(data)

    scalar2float = Scalar2Float()

    output = scalar2float.transform(transformed, minimum, decimal)

    np.testing.assert_allclose(output, expected, rtol=1e-2)



# -*- coding: utf-8 -*-

import numpy as np


def rolling_window_sequences(X, y, index, window_size, target_size, step_size, target_column):
    """Create rolling window sequences out of time series data.

    The function creates an array of input sequences and an array of target sequences by rolling
    over the input sequence with a specified window.
    Optionally, certain values can be dropped from the sequences.

    Args:
        X (ndarray):
            N-dimensional input sequence to iterate over.
        y (ndarray):
            N-dimensional target sequence to iterate over.
        index (ndarray):
            Array containing the index values of X.
        window_size (int):
            Length of the input sequences.
        target_size (int):
            Length of the target sequences.
        step_size (int):
            Indicating the number of steps to move the window forward each round.
        target_column (int):
            Indicating which column of X is the target.

    Returns:
        ndarray, ndarray, ndarray, ndarray:
            * input sequences.
            * target sequences.
            * first index value of each input sequence.
            * first index value of each target sequence.
    """
    out_X = list()
    out_y = list()
    X_index = list()
    y_index = list()
    target = y[:, target_column]

    start = 0
    max_start = len(X) - window_size - target_size + 1
    while start < max_start:
        end = start + window_size

        out_X.append(X[start:end])
        out_y.append(target[end:end + target_size])
        X_index.append(index[start])
        y_index.append(index[end])
        start = start + step_size

    return np.asarray(out_X), np.asarray(out_y), np.asarray(X_index), np.asarray(y_index)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `sigllm` package."""

import unittest

# from sigllm import sigllm


class TestSigllm(unittest.TestCase):
    """Tests for `sigllm` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""
        self.assertTrue(True)
